# $Id: MoviePosterDB.pm 3655 2010-01-12 13:03:01Z chris $

=head1 NAME

WebService::MoviePosterDB - OO Perl interface to the movie poster database MoviePosterDB.

=head1 SYNOPSIS

    use WebService::MoviePosterDB;

    my $ws = WebService::MoviePosterDB->new(cache => 1, cache_exp => "12h");

    my $movie = $ws->search(imdb_code => "0114814", width => 300);

    print $movie->title(), ": \n\n";
    print $movie->page(), "\n\n";

    foreach ( @{$movie->posters()} ) {
        print $_->image_location(), "\n";
    }

=head1 DESCRIPTION

WebService::MusicBrainz is an object-oriented interface to MoviePosterDB.  It can 
be used to retrieve artwork for IMDB titles.

=cut

package WebService::MoviePosterDB;

use strict;
use warnings;

our $VERSION = '0.06';

use Cache::FileCache;

use File::Spec::Functions qw(tmpdir);

use JSON;
use LWP::UserAgent;
use URI;

use WebService::MoviePosterDB::Movie;

=head1 METHODS

=head2 new(%opts)

Constructor.

%opts can contain:

=over 4

=item cache - Whether to cache responses.  Defaults to true

=item cache_root - The root dir for the cache.  Defaults to tmpdir();

=item cache_exp - How long to cache responses for.  Defaults to "1h"

=back

=cut

sub new {
    my $class = shift;
    my %args = @_;
    my $self = {};
    
    bless $self, $class;
    
    $self->{'api_key'} = $args{'api_key'} || "demo";
    $self->{'api_secret'} = $args{'api_secret'} || "demo";
    $self->{'api_version'} = $args{'api_version'} || defined $args{'api_key'} ? 1 : 0;

    $self->{'_cache_root'} = $args{'cache_root'} || tmpdir();
    $self->{'_cache_exp'} = $args{'cache_exp'} || "1h";
    $self->{'_cache'} = defined $args{'cache'} ? $args{'cache'} : 1;

    if ($self->{'_cache'}) {
	$self->{'_cacheObj'} = Cache::FileCache->new( {'cache_root' => $self->{'_cache_root'}, 'namespace' => "WebService-MoviePosterDB", 'default_expires_in' => $self->{'_cache_exp'}} );
    }

    $self->{'_useragent'} = LWP::UserAgent->new();
    $self->{'_useragent'}->env_proxy();
    $self->{'_useragent'}->agent("WebService::MoviePosterDB/$VERSION");

    return $self;
}


=head2 search(%args)

Accesses MoviePosterDB and returns a WebService::MoviePosterDB::Movie object.

%args can contain:

=over 4

=item imdb_code - IMDB code for the title

=item title - Name of the title

=item width - Image width for returned artwork

=back

=cut

# In the fullness of time, the intention is to use the proper MoviePosterDB api, 
# accessed via http://api.movieposterdb.com/json, but until the mechanism for
# generating the secret hash is known, we use the old api, 
# http://api.movieposterdb.com/json.inc.php, and translate the parameters.

sub search {
    my $self = shift;
    my %args = @_;

    my $legacy = $self->{'api_version'} == 0;

    my %_args;

    if (!$legacy) {
	$_args{'api_key'} = $self->{'api_key'};
	$_args{'secret'} = $self->_get_secret($self->{'api_secret'}, $args{'imdb_code'}, $args{'title'});
    }
    
    foreach (keys %args) {
	my ($k, $v) = ($_, $args{$_});
	    
	# Fixup to make json.inc.php look like json
	if ($k eq "imdb_code") {
	    if ($legacy) { $k = "imdb"; }
	    $v = sprintf("%d", $v); # Remove any leading zeroes
	} elsif ($k eq "title") {
	    $v = lc($v);
	} elsif ($k eq "width") {
	    # Nothing
	} else {
	    next;
	}

	$_args{$k} = $v;

    }

    my $uri = URI->new();
    $uri->scheme("http");
    $uri->host("api.movieposterdb.com");
    $uri->path($legacy ? "json.inc.php": "json");
    my @_args = map {$_, $_args{$_}} sort keys %_args;
    foreach (@_args) { utf8::encode($_); }
    $uri->query_form(@_args);

    my $json = JSON->new()->decode($self->_get_page($uri->as_string()));

    return WebService::MoviePosterDB::Movie->_new($json);


}

# Quick hack pending details of hashing algorithm
sub _get_secret {
    my $self = shift;
    my $api_secret = shift;
    my $imdb_code = shift;
    my $title = shift;

    my %_args;
    $_args{'type'} = "JSON";
    $_args{'secret'} = $api_secret;
    if (defined $imdb_code) { $_args{'imdb_code'} = sprintf("%d", $imdb_code); }
    if (defined $title) { $_args{'title'} = lc($title); }

    my $uri = URI->new();
    $uri->scheme("http");
    $uri->host("api.movieposterdb.com");
    $uri->path("console");
    my @_args = map {$_, $_args{$_}} sort keys %_args;
    foreach (@_args) { utf8::encode($_); }
    $uri->query_form(@_args);

    my $content = $self->_get_page($uri->as_string());

    my ($secret) = $content =~ m/secret=([0-9a-f]{12})/ or die "Failed to extract api secret";

    return $secret;

}

sub _get_page {
    my $self = shift;
    my $url = shift;

    my $content;

    if ($self->{'_cache'}) {
	$content = $self->{'_cacheObj'}->get($url);
    }

    if (! defined $content) {
	my $response = $self->{'_useragent'}->get($url);
    
	if($response->code() ne "200") {
	    die "URL (", $url, ") Request Failed - Code: ", $response->code(), " Error: ", $response->message(), "\n";
	}

	$content = $response->decoded_content();

	if ($self->{'_cache'}) {
	    $self->{'_cacheObj'}->set($url, $content);
	}
    }

    return $content;
}

1;
